<cfcomponent>

	<cffunction name="process" output="false" access="public" returntype="any" hint="I process a workflow request">
		<cfargument name="ActionData" type="any" required="true"/>
		
		<cfscript>
		var x = "";
		var y = "";
		commands = ActionData.getCommandSet();
		If( isArray( commands ) ){
			
			for( x = 1; x lte arrayLen( commands ); x = x + 1){
				
				for( y in commands[x] ){
					switch( y ){
						
					    case 'newDocument':{
					    		newDocument( commands[x][y], arguments.ActionData );
					      break;
					    }
					    case 'provisionTemplate':{
					    		provisionTemplate( commands[x][y], arguments.ActionData );
					      break;
					    }
					    case 'sendEmail':{
					    		sendEmails( commands[x][y], arguments.ActionData );
					      break;
					    }
					    case 'changeQueue':{
					    		changeQueue( commands[x][y], arguments.ActionData );
					      break;
					    }
					    case 'addHistory':{
					    		addHistory( commands[x][y], arguments.ActionData );
					      break;
					    }
					    case 'startTime':{
					    		startTime( commands[x][y], arguments.ActionData );
					      break;
					    }
					    case 'endTime':{
					    		endTime( commands[x][y], arguments.ActionData );
					      break;
					    }
					    case 'markSaved':{
					    		markSaved( commands[x][y], arguments.ActionData );
					      break;
					    }
					}			
				}
				
			}			
		}
		
		</cfscript>

	</cffunction>
	
	<cffunction name="markSaved" output="false" access="public" returntype="void" hint="I mark a document as saved">
		<cfargument name="mode" type="string" required="true"/>
		<cfargument name="ActionData" type="any" required="true"/>
		
		<cfset getWorkFlowDAO().saveDocument( arguments.ActionData.getDocumentID() ) />

	</cffunction>
		
	<cffunction name="assignOwner" output="false" access="private" returntype="void" hint="I assign an owner to a document">
		<cfargument name="mode" type="string" required="true"/>
		<cfargument name="ActionData" type="any" required="true"/>

		<cfset getWorkFlowDAO().assignOwner( arguments.ActionData.getDocumentID(), arguments.ActionData.getUser().getUserID(), arguments.ActionData.getResultQueueName() ) />		
	</cffunction>

	<cffunction name="newDocument" output="false" access="private" returntype="void" hint="I assign an owner to a document">
		<cfargument name="mode" type="string" required="true"/>
		<cfargument name="ActionData" type="any" required="true"/>

		<cfset getWorkFlowDAO().newDocument( arguments.ActionData.getDocumentID(), arguments.ActionData.getUser().getUserID(), arguments.ActionData.getResultQueueName() ) />		
	</cffunction>

	<cffunction name="provisionTemplate" output="false" access="private" returntype="void" hint="I create a new template file">
		<cfargument name="mode" type="string" required="true"/>
		<cfargument name="ActionData" type="any" required="true"/>

		<cfset getWorkFlowDAO().provisionTemplate( arguments.ActionData.getDocumentID() ) />		
	</cffunction>

	<cffunction name="changeQueue" output="false" access="private" returntype="void" hint="I change the queue for a document">
		<cfargument name="mode" type="string" required="true"/>
		<cfargument name="ActionData" type="any" required="true"/>

			<cfset getWorkFlowDAO().changeQueue( arguments.ActionData.getDocumentID(), arguments.ActionData.getResultQueueName() ) >
	</cffunction>

	<cffunction name="addHistory" output="false" access="private" returntype="void" hint="I add a history record">
		<cfargument name="mode" type="string" required="true"/>
		<cfargument name="ActionData" type="any" required="true"/>

		<cfset getWorkFlowDAO().addHistory( arguments.ActionData.getDocumentID(), arguments.ActionData.getUser().getUserID(), arguments.ActionData.getActionTitle(), arguments.ActionData.getEmailBody() ) />	
	</cffunction>

	<cffunction name="startTime" output="false" access="private" returntype="void" hint="I start the time clock for a document">
		<cfargument name="mode" type="string" required="true"/>
		<cfargument name="ActionData" type="any" required="true"/>
	
			<cfif arguments.mode IS  getAppConfig().getConfig('AttorneyRoleCode')>
				<cfset getWorkFlowDAO().startLegalTime( arguments.ActionData.getDocumentID(), arguments.ActionData.getUser().getUserID()  ) />	
			<cfelseif arguments.mode IS  getAppConfig().getConfig('LeaseSubmitterRoleCode')>	
				<cfset getWorkFlowDAO().startFieldTime( arguments.ActionData.getDocumentID(), arguments.ActionData.getUser().getUserID()  ) />	
			</cfif>
	</cffunction>
	
	<cffunction name="endTime" output="false" access="private" returntype="void" hint="I end the time clock for a document">
		<cfargument name="mode" type="string" required="true"/>
		<cfargument name="ActionData" type="any" required="true"/>
		<cfset var emailList = "" />

			<cfif arguments.mode IS  getAppConfig().getConfig('AttorneyRoleCode')>
				<cfset emailList = getDocumentUserService().getAttorneyOrLegalAdminEmails( arguments.ActionData.getDocumentID() ) />
				<cfset getWorkFlowDAO().endLegalTime( arguments.ActionData.getDocumentID() ) />	
			<cfelseif arguments.mode IS  getAppConfig().getConfig('LeaseSubmitterRoleCode')>	
				<cfset emailList = getDocumentUserService().getLeaseSubmitterEmail( arguments.ActionData.getDocumentID() ) />
				<cfset getWorkFlowDAO().endFieldTime( arguments.ActionData.getDocumentID() ) />	
			</cfif>
	</cffunction>

	<cffunction name="sendEmails" output="false" access="private" returntype="void" hint="I send the emails for the end of the process">
		<cfargument name="mode" type="string" required="true"/>
		<cfargument name="ActionData" type="any" required="true"/>
		
			<cfset getEmailService().SendEmail( loadEmails( mode, ActionData ), arguments.ActionData.getEmailSubject(), arguments.ActionData.getEmailBody() ) />
	
	</cffunction>

	<cffunction name="loadEmails" output="false" access="private" returntype="string" hint="I return a list of emails">
		<cfargument name="mode" type="string" required="true"/>
		<cfargument name="ActionData" type="any" required="true"/>
		<cfset var emailList = "" />
		
		<cfswitch expression="#Mode#">
			
			<cfcase value="ByID">
				<cfset emailList = getDocumentUserService().getUserListByID( arguments.ActionData.getUser().getUserID() ) />
			</cfcase>
			<cfcase value="AttorneyOrLegalAdmin">
				<cfset emailList = getDocumentUserService().getAttorneyOrLegalAdminEmails( arguments.ActionData.getDocumentID() ) />
			</cfcase>
			<cfcase value="Lease Submitter">
				<cfset emailList = getDocumentUserService().getLeaseSubmitterEmail( arguments.ActionData.getDocumentID() ) />
			</cfcase>
			<cfcase value="Attorney">
				<cfset emailList = getDocumentUserService().getAttorneyEmails( arguments.ActionData.getDocumentID() )  />
			</cfcase>
			<cfcase value="ByFilter">
				<cfset emailList = getTypeGW().loadUsers( arguments.ActionData.getFilter(), true ) />
				<cfset emailList = ArrayToList(emailList["email"])   />
			</cfcase>
		
		</cfswitch>
		<cfreturn emailList />
	
	</cffunction>

	<cffunction name="getDocumentUserService" access="public" output="false" returntype="any">
		<cfreturn variables.instance.DocumentUserService />
	</cffunction>
	
	<cffunction name="setDocumentUserService" access="public" output="false" returntype="void">
		<cfargument name="DocumentUserService" type="any" required="true" />
		<cfset variables.instance.DocumentUserService = arguments.DocumentUserService />
	</cffunction>

	<cffunction name="getEmailService" access="public" output="false" returntype="any">
		<cfreturn variables.instance.EmailService />
	</cffunction>
	
	<cffunction name="setEmailService" access="public" output="false" returntype="void">
		<cfargument name="EmailService" type="any" required="true" />
		<cfset variables.instance.EmailService = arguments.EmailService />
	</cffunction>

	<cffunction name="getWorkFlowDAO" access="public" output="false" returntype="any">
		<cfreturn variables.instance.WorkFlowDAO />
	</cffunction>
	
	<cffunction name="setWorkFlowDAO" access="public" output="false" returntype="void">
		<cfargument name="WorkFlowDAO" type="any" required="true" />
		<cfset variables.instance.WorkFlowDAO = arguments.WorkFlowDAO />
	</cffunction>
	
	<cffunction name="getAppConfig" access="public" output="false" returntype="any">
		<cfreturn variables.instance.AppConfig />
	</cffunction>
	
	<cffunction name="setAppConfig" access="public" output="false" returntype="void">
		<cfargument name="AppConfig" type="any" required="true" />
		<cfset variables.instance.AppConfig = arguments.AppConfig />
	</cffunction>

	<cffunction name="getTypeGW" access="public" output="false" returntype="any">
		<cfreturn variables.instance.TypeGW />
	</cffunction>
	
	<cffunction name="setTypeGW" access="public" output="false" returntype="void">
		<cfargument name="TypeGW" type="any" required="true" />
		<cfset variables.instance.TypeGW = arguments.TypeGW />
	</cffunction>

</cfcomponent>